################################################################################
#
#		SIMULATION OF ELECTRICITY TRADE BETWEEN TWO COUNTRIES
#
# Use realistic data for electricity load and renewables supply 
# to compute gains from trade in different scenarios
#
# Static efficiency: gains from connecting 2 countries,
#		with fixed power plant park
# Dynamic efficiency: gains from connecting 2 countries,
#		optimizing the power plant park according to % allowed trade
#
# Author: Michele Peruzzi
# Date: April, 2013
# Contact: michele.peruzzi@bruegel.org peruzzi@gmail.com
#
################################################################################


# Packages needed for plots: uncomment if you need to install them
#install.packages("ggplot2")
#install.packages("gridExtra")

# Set the current directory to the location of 'main.r'
# If you copy+paste from Windows, remember to change \ into /

workingdir <- "/home/bruegel/Dropbox/Work/cakes/energy infrastructure/benefits from trade/de-de/"

setwd(workingdir)

# Load source code for custom function
source("source/source_functions_de.r")

# Realistic data: peak load (in MW)
peak_A <- 25650  # west = amprion
peak_B <- 17700		# center = tennet

# Realistic data: installed capacity (in MW)
# Renewables
rens_A <- 13000 # amprion
rens_B <- 23000
# Fossil fuels
gas_A <- 10600 # amprion
gas_B <- 7600
coal_A <- 22600 # amprion
coal_B <- 7100
# Nuclear
nuclear_A <- 3900
nuclear_B <- 5500

# Variable costs: from literature (in EUR/MW)
vc_res <- 0
vc_gas <- 35
vc_coal <- 21
vc_nuclear <- 10
cost_over <- 20000 	# EUR for 1 MW of unfulfilled demand

# Fixed costs: from literature (in EUR/MW year)
fc_res <- 120000
fc_gas <- 40000
fc_coal <- 100000
fc_nuclear <- 190000

############################ SIMULATION PARAMETERS #############################

# Renewables multiplier. Ex. if 2, then consider 2x RES
res_conf <- c(1,2)

# Percentage of trade available
# 	- calculated as % of total installed capacity in smaller country
#	- put large number for the infinite-trade case (10 = 1000% of capacity)
trade_conf <- c(0, 0.05, 10)

############ STATIC EFFICIENCY
# stored in a list. use ls(static[[1]]) to have a list of output variables

static <- list()
for(td in 1:length(trade_conf)){
	res_A <- rens_A
	res_B <- rens_B
	trade_perc <- trade_conf[td]
	source("source/source_other_de.r")
	static[[td]] <- get_data(pp=c(coal_A, coal_B, gas_A, gas_B), 
							capinst=c(rens_A, rens_B, nuclear_A, nuclear_B))
}

############ DYNAMIC EFFICIENCY
# stored in a list. use ls(dynamic[[1]]) to have a list of output variables

dynamic <- list()
for(r in 1:length(res_conf))
{
	for(td in 1:length(trade_conf))
	{
		weare <- (r-1)*length(trade_conf) + td
        
		res_A <- rens_A * res_conf[r]
		res_B <- rens_B * res_conf[r]
		trade_perc <- trade_conf[td]
		
		source("source/source_other_de.r")
		
		start <- proc.time()
		try <- optim(c(3000,3000,3000,3000), opt_power_plant_park)
		end <- proc.time()
		print((end-start)[3])

		dynamic[[weare]] <- get_data(pp=try$par, 
									capinst=c(res_A, res_B, nuclear_A, nuclear_B))

		print(paste0(weare,"th optim done"))
	}
}

# the optimal power plant park is stored in dynamic[[s]]$par 
# where s is the number of the simulation

cost_results_zone <- rbind(1, static[[2]]$cost/static[[1]]$cost, 
        static[[3]]$cost/static[[1]]$cost, 1,
        dynamic[[2]]$cost/dynamic[[1]]$cost, 
        dynamic[[3]]$cost/dynamic[[1]]$cost, 1,
        dynamic[[5]]$cost/dynamic[[4]]$cost, 
        dynamic[[6]]$cost/dynamic[[4]]$cost)
        
cost_results_sum <- rbind(1, sum(static[[2]]$cost)/sum(static[[1]]$cost), 
        sum(static[[3]]$cost)/sum(static[[1]]$cost), 1,
        sum(dynamic[[2]]$cost)/sum(dynamic[[1]]$cost), 
        sum(dynamic[[3]]$cost)/sum(dynamic[[1]]$cost), 1,
        sum(dynamic[[5]]$cost)/sum(dynamic[[4]]$cost), 
        sum(dynamic[[6]]$cost)/sum(dynamic[[4]]$cost))

average_prices <- rbind(calcprice(static[[1]]$usage, static[[1]]$supply, 0), 
                        calcprice(static[[2]]$usage, static[[2]]$supply, 2160), 
                        calcprice(static[[3]]$usage, static[[3]]$supply, 999999),
                        
                        calcprice(dynamic[[1]]$usage, dynamic[[1]]$supply, 0), 
                        calcprice(dynamic[[2]]$usage, dynamic[[2]]$supply, 2160), 
                        calcprice(dynamic[[3]]$usage, dynamic[[3]]$supply, 999999),
                        
                        calcprice(dynamic[[4]]$usage, dynamic[[4]]$supply, 0), 
                        calcprice(dynamic[[5]]$usage, dynamic[[5]]$supply, 3155), 
                        calcprice(dynamic[[6]]$usage, dynamic[[6]]$supply, 999999))
                        
colnames(average_prices) <- c("Avg P Zone A - West", "Avg P Zone B - Center")

rownames(cost_results_zone) <- rownames(cost_results_sum) <- c("Static", "Static 5%", "Static Full", "Dynamic 1x", "Dynamic 1x 5%", "Dynamic 1x Full", "Dynamic 2x", "Dynamic 2x 5%", "Dynamic 2x Full")
cost_results <- cbind(cost_results_zone, cost_results_sum, average_prices)
