freq <- 8748

# function to update supply and demand
# inputs: country, resource, quantity available, initial demand
# outputs: country, residual quantity available, residual demand
#
# format/type of variables: 
# country, resource = string [1]
# q_avail, init_d = numeric [freq, resource, country]
# usage, resid_q_avail, resid_d = numeric [freq]

library(scales)
library(ggplot2)
library(gridExtra)

update_qd <- function(country, resource, q_avail, init_d){
    resid_d <- c()
    usage <- rep(0, freq)
    usage <- apply(cbind(q_avail[,resource,country],init_d[,country]), 1, min)
    resid_d <- init_d[,country] - usage
    resid_q_avail <- q_avail[,resource,country] - usage 

    out <- data.frame(usage = numeric(freq), resid_q = numeric(freq), resid_d = numeric(freq))
    out$usage <- usage
    out$resid_q <- resid_q_avail
    out$resid_d <- resid_d
return(out)
}

# function to update trade
# inputs: country, resource, limit to trade (absolute), quantity available
# outputs: quantity available for trade
#
# format/type of variables: 
# country, resource = string [1]
# limit = numeric [country]
# q_avail = numeric [freq]
# trade_available = numeric [freq]


update_trade <- function(country, limit, q_avail){

    trade_available <- q_avail
    for(t in 1:freq){
        if(trade_available[t] > limit[t,country]){ 
            trade_available[t] <- limit[t,country]
        }
    }
    out <- data.frame(trade_avail = numeric(freq))
    out$trade_avail <- trade_available
return(out)
}

opt_power_plant_park <- function(pp){
    return(opt_pplant_base(power_plants=pp, 
							installed=c(res_A, res_B, nuclear_A, nuclear_B), 
							optimize=1))
}

get_data <- function(pp, capinst){
	return(opt_pplant_base(power_plants=pp, installed=capinst, optimize=0))
}


opt_pplant_base <- function(power_plants, installed, optimize){

	names(power_plants) <- c("Coal_A","Coal_B", "Gas_A","Gas_B")
	names(installed) <- c("RES_A","RES_B","Nuclear_A","Nuclear_B")
	residual_demand <- demand

    # Supply values:
    supply <- array(0, dim=c(freq,4,2), dimnames=list(1:freq, resources, countries))
    supply[,"RES",] <- cbind(installed["RES_A"]*res[,1], installed["RES_B"]*res[,2])
    supply[,"Gas",] <- cbind(power_plants["Gas_A"]*gas[,1], power_plants["Gas_B"]*gas[,2])
	supply[,"Coal",] <- cbind(power_plants["Coal_A"]*coa[,1], power_plants["Coal_B"]*coa[,2])
    supply[,"Nuclear",] <- cbind(installed["Nuclear_A"]*nuc[,1], installed["Nuclear_B"]*nuc[,2])

    # reset available imports and usage of resources
    avail_import <- array(0, dim=c(freq,4,2), dimnames=list(1:freq, resources, countries))
    usage <- array(0, dim=c(freq,4*2,2), dimnames=list(1:freq, resources_trade, countries))

    # establish values
   
    for(resource in resources){
        for(c in 1:2){
            r <- update_qd(countries[c], resource, supply, residual_demand)
            usage[,resource,countries[c]] <- r$usage
            residual_demand[,countries[c]] <- r$resid_d
        }
        for(c in 1:2){
            exportable <- update_trade(countries[-c], trlim, supply[,resource,countries[-c]] - usage[,resource,countries[-c]])
            avail_import[,resource,countries[c]] <- exportable$trade_avail
            r <- update_qd(countries[c], resource, avail_import, residual_demand)
            usage[,paste(resource,"_import",sep=""),countries[c]] <- r$usage
            trlim[,countries[-c]] <- trlim[,countries[-c]] - r$usage
            residual_demand[,countries[c]] <- r$resid_d

        }
    }
    
    cost <- c(0,0)
  
    cost[1] <- c(installed["RES_A"],installed["Nuclear_A"],power_plants["Coal_A"],power_plants["Gas_A"])%*%fc[,"Zone A"] + sum(tcrossprod(vc[,"Zone A"],usage[,,"Zone A"])) + exceed*sum(residual_demand[,"Zone A"])
    cost[2] <- c(installed["RES_B"],installed["Nuclear_B"],power_plants["Coal_B"],power_plants["Gas_B"])%*%fc[,"Zone B"] + sum(tcrossprod(vc[,"Zone B"],usage[,,"Zone B"])) + exceed*sum(residual_demand[,"Zone B"])
    
    if(length(power_plants[power_plants>=0]) <4){ cost[1] <- Inf}
	names(cost) <- countries
    
	if(optimize==1){
		return(sum(cost))
	} else {
		out <- list(residual_demand=residual_demand, 
					usage=usage,
					cost=cost, 
					supply=supply, 
					par=power_plants)
		return(out)
	}
}
#  

supply_plot <- function(capacity, limits, vcs, country, stringtitle){
    
	#if(sum(limits)==0)
	
    data <- data.frame(cap=capacity, vc=vcs)
	#print(data$cap)
	#data$cap[data$cap==0] <- 1
	total_capacity <- sum(data$cap)
	
	gr <- ggplot(data) + 
    geom_rect(aes(xmin=0,xmax=cap[1],ymin=0,ymax=39+vc[1]), fill="darkgreen", alpha=0.3) + 
    geom_rect(aes(xmin=(1+cap[1]),xmax=(cap[1]+cap[2]),ymin=0,ymax=39+vc[2]), fill="darkblue", alpha=0.3) + 
    geom_rect(aes(xmin=(1+cap[1]+cap[2]),xmax=(cap[1]+cap[2]+cap[3]),ymin=0,ymax=39+vc[3]), fill="darkred", alpha=0.3) +
	geom_rect(aes(xmin=(1+cap[1]+cap[2]+cap[3]),xmax=(cap[1]+cap[2]+cap[3]+cap[4]),ymin=0,ymax=39+vc[4]), fill="grey80", alpha=0.5) +	
    scale_y_continuous(name="Price") +
    scale_x_continuous(label=comma,name="Quantity (MW)", limits=limits)+ ggtitle(paste0(ifelse(stringtitle!="","Optimized ", ""),"Energy supply, ",country,ifelse(stringtitle!="",", ", ""), stringtitle)) + theme(axis.text.y=element_blank())
	
	if(capacity[1]>1000){ 
		gr <- gr + annotate("text", label = "RES", x = 0.5*(capacity[1]), y = (39+vcs[1])/2, size = 6) 
		} else {}
		
	if(capacity[2]>1000){ 
		gr <- gr + annotate("text", label = "Nuclear", x = 0.5*(1+capacity[1]+capacity[1]+capacity[2]), y = (39+vcs[1]+vcs[2])/2, size = 6) 
		} else {}
		
	if(capacity[3]>1000){ 
		gr <- gr + annotate("text", label = "Coal", x = 0.5*(1+capacity[1]+capacity[2]+capacity[1]+capacity[2]+capacity[3]), y = (39+vcs[1]+vcs[2]+vcs[3])/2, size = 6) 
		} else {}
		
	if(capacity[4]>1000){ 
		gr <- gr + annotate("text", label = "Gas", x = 0.5*(1+capacity[1]+capacity[2]+capacity[3]+capacity[1]+capacity[2]+capacity[3]+capacity[4]), y = (39+vcs[1]+vcs[2]+vcs[3]+vcs[4])/2, size = 6) 
		} else {}
	
return(gr)
}


drawplots <- function(trade_results, installed, from_time=200, to_time=400, suplim=0, onegraph=TRUE, stringtitle){
	if(length(suplim)==1){
		suplim <- matrix(0, ncol=2)
	}
    country <- countries
    print(country)
    for(c in 1:2){
        assign(paste("data",c,sep=""),data.frame(time=from_time:to_time,
                                                demand=demand[from_time:to_time,country[c]], 
                                                foreign_demand=demand[from_time:to_time,country[-c]],
                                                supply_res=trade_results$supply[from_time:to_time,"RES",country[c]], 
                                                usage_res=trade_results$usage[from_time:to_time,"RES",country[c]], 
                                                import_res=trade_results$usage[from_time:to_time,"RES_import",country[c]], 
                                                supply_nuc=trade_results$supply[from_time:to_time,"Nuclear",country[c]], 
                                                usage_nuc=trade_results$usage[from_time:to_time,"Nuclear",country[c]], 
                                                import_nuc=trade_results$usage[from_time:to_time,"Nuclear_import",country[c]],
												supply_coal=trade_results$supply[from_time:to_time,"Coal",country[c]], 
                                                usage_coal=trade_results$usage[from_time:to_time,"Coal",country[c]], 
                                                import_coal=trade_results$usage[from_time:to_time,"Coal_import",country[c]],											
                                                supply_gas=trade_results$supply[from_time:to_time,"Gas",country[c]], 
                                                usage_gas=trade_results$usage[from_time:to_time,"Gas",country[c]], 
                                                import_gas=trade_results$usage[from_time:to_time,"Gas_import",country[c]]))
    }

    for(c in 1:2){
    assign(paste("demand",c,sep=""), ggplot(get(paste("data",c,sep="")), aes(time)) + 
                                    geom_line(aes(y=demand), color="black", size=1) +
                                    #geom_line(aes(y=foreign_demand), color="grey50") + 
                                    ggtitle(paste("Demand,",country[c],sep=" ")) +
									#theme(axis.title.y=element_blank()) +
                                    scale_y_continuous(name="MW", limits=c(0,max(demand))) +
									scale_x_continuous(name="Time in hours"))
                                    
    assign(paste("s",c,sep=""), ggplot(get(paste("data",c,sep="")), aes(time)) + 
                                    geom_line(aes(y=supply_res), color="darkgreen") +
                                    #geom_line(aes(y=supply_gas), color="grey70") + 
									#geom_line(aes(y=supply_coal), color="darkred") +
                                    #geom_line(aes(y=supply_nuc), color="darkblue") + 
                                    ggtitle(paste("RES production over time,",country[c],sep=" ")) +
									#theme(axis.title.y=element_blank()) +
                                    scale_y_continuous(name="MW", limits=c(0,max(trade_results$supply))) +
									scale_x_continuous(name="Time in hours"))
                                    
    assign(paste("u",c,sep=""), ggplot(get(paste("data",c,sep="")), aes(time)) + 
                                    geom_line(aes(y=usage_res), color="darkgreen") +
                                    geom_line(aes(y=usage_gas), color="grey70") + 
									geom_line(aes(y=usage_coal), color="darkred") +
                                    geom_line(aes(y=usage_nuc), color="darkblue") + 
                                    ggtitle(paste("Simulated Usage,",country[c],sep=" "))+
									#theme(axis.title.y=element_blank()) +
                                    scale_y_continuous(name="MW", limits=c(0,max(trade_results$usage))) +
									scale_x_continuous(name="Time in hours"))
                                    
    assign(paste("t",c,sep=""), ggplot(get(paste("data",c,sep="")), aes(time)) + 
                                    geom_line(aes(y=import_res), color="darkgreen") +
                                    geom_line(aes(y=import_gas), color="grey70") +
									geom_line(aes(y=import_coal), color="darkred") + 									
                                    geom_line(aes(y=import_nuc), color="darkblue") + 
                                    ggtitle(paste("Simulated Imports,",country[c],sep=" "))+
									#theme(axis.title.y=element_blank()) +
                                    scale_y_continuous(name="MW", limits=c(0,max(trade_results$usage))) +
									scale_x_continuous(name="Time in hours"))
                                                
                                                
    }
    su1 <- supply_plot(capacity=c(installed[c(1,3)], trade_results$par[c(1,3)]), limits=suplim[1,], vcs=c(vc_res,vc_nuclear,vc_coal,vc_gas), "Zone A", stringtitle=stringtitle)
    su2 <- supply_plot(capacity=c(installed[c(2,4)], trade_results$par[c(2,4)]), limits=suplim[2,], vcs=c(vc_res,vc_nuclear,vc_coal,vc_gas), "Zone B", stringtitle=stringtitle)
    if(onegraph)
	{
		grid.arrange(su1, su2, demand1, demand2, s1, s2, u1, u2, t1, t2, ncol=2)
		
	}
	else
	{
		graphlist <- list(su1, su2, demand1, demand2, s1, s2, u1, u2, t1, t2)
		names(graphlist) <- c("SupplyfA", "SupplyfB", "DemandA", "DemandB", "SupplyA", "SupplyB", "UsageA", "UsageB", "ImportA", "ImportB")
		graphlist
	}
}


average_price <- function(resource_usage){
    imports <- c(2,4,6,8)
    price <- matrix(0, ncol=2, nrow=freq)
    for(i in 1:freq){
        price[i,] <- c(max(vc[,1]*(resource_usage[i,,"Zone A"] > 0)), max(vc[,2]*(resource_usage[i,,"Zone B"] > 0)))
        maxa <- max(which(resource_usage[i,,"Zone A"] > 0))
        maxa[maxa==-Inf] <- 0
        maxb <- max(which(resource_usage[i,,"Zone B"] > 0))
        maxb[maxb==-Inf] <- 0
        if( maxa %in% imports){
            price[i,2] <- vc[maxa-1,2]
        
        } else {
            if(maxa > maxb){
                price[i,2] <- vc[maxa-2,2]
            }
        }
        if( maxb %in% imports){
            price[i,1] <- vc[maxb-1,1]
        }else {
            if(maxb > maxa){
                price[i,1] <- vc[maxb-2,1]
            }
        }
    }
return(price)
}


calcprice <- function(usage, supply, limit){
    local <- c(1,3,5,7)
    imports <- c(2,4,6,8)
    available <- array(0, dim=dim(usage), dimnames=dimnames(usage))
    connection <- rowSums(usage[,imports,1] + usage[,imports,2]) < limit
    
    available[,local,1] <- supply[,,1] - usage[,local,1] - usage[,imports,2]
    available[,imports,1] <- ifelse(connection, supply[,,2] - usage[,local,2] - usage[,imports,1], rep(0,freq))
    
    available[,local,2] <- supply[,,2] - usage[,local,2] - usage[,imports,1]
    available[,imports,2] <- ifelse(connection, supply[,,1] - usage[,local,1] - usage[,imports,2], rep(0,freq))
    
    a <- apply(available[,,1], 1, function(x) ifelse(min(which(x>0))!=Inf,min(which(x>0)),length(vc)))
    b <- apply(available[,,2], 1, function(x) ifelse(min(which(x>0))!=Inf,min(which(x>0)),length(vc)))

    c <- apply(usage[,,1], 1, function(x) ifelse(max(which(x>0))!=-Inf,max(which(x>0)),1))
    d <- apply(usage[,,2], 1, function(x) ifelse(max(which(x>0))!=-Inf,max(which(x>0)),1))
    
    #vca <- c(vc,cost_over)
    
    results <- cbind(vc[a], vc[c], vc[b], vc[d], connection)
    priceb <- pricea <- rep(0,freq)
    for(i in 1:freq){
        if(connection[i]){
            if(results[i,2] > results[i,4]){
                if(results[i,2]==0.1+results[i,3]){
                    pricea[i] <- results[i,2]
                } else {
                    pricea[i] <- results[i,1]
                }
                priceb[i] <- results[i,2]-0.1
            } else {
                if(results[i,2] < results[i,4]){
                    
                    pricea[i] <- results[i,4]-0.1
                    #priceb[i] <- results[i,3]
                    if(results[i,4]==0.1+results[i,1]){
                        priceb[i] <- results[i,4]
                    } else {
                    priceb[i] <- results[i,3]
                    }
                } else {
                    pricea[i] <- priceb[i] <- results[i,2]
                }        
            }
        } else 
        {
            pricea[i] <- results[i,1]
            priceb[i] <- results[i,3]
        }
    }
    
    
    colnames(results) <- c("Available, A",  "Last used, A", "Available, B","Last used, B", "Can trade")
    r <- list(results = results, price = cbind(pricea, priceb))
    return(r$price)
#return(apply(r$price,2,mean))
}



